Node-Broker
===========

Subscriber and Publisher do not know about each other; they only know Node-Broker. It's like a cross-module messaging and processing system.

Installation
------------

Broker can be used on both server and/or client.

### Server (Node.js)
    
    npm install broker
	
### Client (Browser)

	<script src="broker.js" type="text/javascript"></script>
	
In the browser, the broker object is a property of the global window object. Thus, broker can be accessed anywhere.

Justification
-------------

Since commands are available everywhere, the broker can be seen as a global command center. Why should you ever want to do this? The simple reason is because it allows you to create modules that simply provide and require data from the broker, meaning that any other module can work with the original module by simply providing the required commands to the broker.

Example 1 (Node.js)
---------

### Module1.js

    var broker = require('broker');
    
    broker.handle('getAppName', function(cb) {
        cb('BrokerTest');
    });

### Module2.js

    var broker = require('broker');
    
    broker.exec('getAppName', function(name) {
        require('util').debug('app name is ' + name);
    });
    
__Note:__ a handle must be set before it is attempted to be executed, or an error will be thrown.

Example 2 (Node.js)
---------

See https://github.com/tommedema/node-broker/tree/master/examples/simple for a simple example.

Example 3: control flow (Node.js)
-----------------------

See https://github.com/tommedema/node-broker/tree/master/examples/flow for a control flow example.

Example 4: browser usage, prevent global pollution using node-broker
-----------------------

See https://github.com/tommedema/node-broker/tree/master/examples/browser for a browser example.

Control Flow with multiple callbacks
---------

You can also use a control flow format to chain commands and continue when all callbacks have been fired. You can chain execution commands by passing a command object as the first parameter and a done callback as the second. The done callback will be called once all commands' callbacks have been executed. If there are no callbacks, this function will be called immediately. Each command can have a callback directly by setting it to a function, or have an array with arguments. In the latter case, the last function in the array will be considered as the callback. An example:

    broker.exec({
        'getAppName' : function(appName) {
            util.debug('app name is: ' + appName);
        },
        'setAppName' : ['NewName', function(result) {
            util.debug('got result from set app name: ' + result);
        }],
        'doSomethingCool' : null //this one does not need any arguments
    }, function() {
        util.debug('all commands have been executed!');
    });

Multiple Handles in a Single Call
---------------------------------

You can set multiple handles within a single call, like so:

    broker.handle({
        'setAppName' : function(_appName) {
            appName = _appName;
        },
        'getAppName' : function(cb) {
            cb(appName);
        },
        'doSomethingCool' : function() {
            //lets do something amazing here
        }
    });

Namespacing
-----------

You are responsible for your own namespacing, this can be done by simply prefixing your command with a namespace:
    
    broker.handle('chat#remClient', function(name) { });
    
And to execute it:
    
    broker.exec('chat#remClient', 'Tom');
    
The above example uses the remClient command in the chat namespace. Note that this command can still be used anywhere, it simply needs to be prefixed with the namespace.

Dynamic Handles
----------------

You can remove a handle (and set a new one) using broker.removeHandle('someCommand');